﻿using DocumentFormat.OpenXml.InkML;
using IRaCIS.Core.Infrastructure;
using Microsoft.AspNetCore.Diagnostics;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;


namespace IRaCIS.Core.Application.BusinessFilter;


/// <summary>
/// minimal api 生效，但是传统控制器，没生效
/// 参考处理链接： https://learn.microsoft.com/zh-cn/aspnet/core/fundamentals/error-handling?view=aspnetcore-8.0
/// </summary>
public class GlobalExceptionHandler(IStringLocalizer _localizer, ILogger<GlobalExceptionHandler> _logger) : IExceptionHandler
{

    public ValueTask<bool> TryHandleAsync(
        HttpContext httpContext,
        Exception exception,
        CancellationToken cancellationToken)
    {

        httpContext.Response.StatusCode =StatusCodes.Status200OK;

        httpContext.Response.ContentType = "application/json";

        if (exception.GetType().Name == "DbUpdateConcurrencyException")
        {
            //---并发更新，当前不允许该操作

            httpContext.Response.WriteAsJsonAsync(ResponseOutput.NotOk(_localizer["ProjectException_ConcurrentUpdateNotAllowed"] + exception.Message));
        }

        if (exception.GetType() == typeof(BusinessValidationFailedException))
        {
            var error = exception as BusinessValidationFailedException;

            var info = string.Empty;

            if (!string.IsNullOrWhiteSpace(error!.LocalizedKey) && StaticData.Localizer_Dev_Dic.ContainsKey(error!.LocalizedKey))
            {
                info = $"[{error!.LocalizedKey}]:{StaticData.Localizer_Dev_Dic[error!.LocalizedKey]}";
            }

            httpContext.Response.WriteAsJsonAsync(ResponseOutput.NotOk(exception.Message, "", error!.Code, localizedInfo: info));



            //warning 级别记录
            //_logger.LogWarning($"[{error!.LocalizedKey}]:{StaticData.Log_Locoalize_Dic[error!.LocalizedKey]}");
        }
        else if (exception.GetType() == typeof(QueryBusinessObjectNotExistException))
        {
            httpContext.Response.WriteAsJsonAsync(ResponseOutput.NotOk(exception.Message, ApiResponseCodeEnum.DataNotExist));
        }
        else
        {
            httpContext.Response.WriteAsJsonAsync(ResponseOutput.NotOk(_localizer["Project_ExceptionContactDeveloper"] + (exception.InnerException is null ? (exception.Message )
            : (exception.InnerException?.Message)), ApiResponseCodeEnum.ProgramException));
         
        }

        var errorInfo = $"Exception: {exception.Message}[{exception.StackTrace}]" + (exception.InnerException != null ? $" InnerException: {exception.InnerException.Message}[{exception.InnerException.StackTrace}]" : "");

        _logger.LogError(errorInfo);

        // Return false to continue with the default behavior
        // - or - return true to signal that this exception is handled
        return ValueTask.FromResult(false);
    }
}
