﻿using Microsoft.AspNetCore.Http;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IRaCIS.Core.Application.BusinessFilter;

public class ModelValidationEndpointFilter(IStringLocalizer _localizer) : IEndpointFilter
{
    public async ValueTask<object?> InvokeAsync(EndpointFilterInvocationContext context, EndpointFilterDelegate next)
    {
        var httpContext = context.HttpContext;

        //// 1. 获取路由参数并验证
        //foreach (var routeValue in httpContext.Request.RouteValues)
        //{
        //    {
        //        return Results.BadRequest(new { ErrorMessage = $"Invalid format for {routeValue.Key}." });
        //    }
        //}

        //// 2. 获取查询参数并验证
        //foreach (var queryValue in httpContext.Request.Query)
        //{
        //    {
        //        return Results.BadRequest(new { ErrorMessage = $"Invalid format for {queryValue.Key}." });
        //    }
        //}


        // 获取请求中的模型参数
        var model = context.Arguments.FirstOrDefault(a => a != null && a.GetType().IsClass);

        // 仅当模型存在时执行验证
        if (model != null)
        {
            var validationResults = new List<ValidationResult>();
            var validationContext = new ValidationContext(model);
            bool isValid = Validator.TryValidateObject(model, validationContext, validationResults, true);

            if (!isValid)
            {
                var validationErrors = validationResults.Select(vr => vr.ErrorMessage).ToArray();
                return Results.Json(new
                {
                    ErrorMessage = _localizer["ModelAction_InvalidAPIParameter"],
                    Errors = validationErrors
                }, statusCode: StatusCodes.Status400BadRequest);
            }
        }

        // 验证通过，继续执行
        return await next(context);
    }
}
