﻿using System;
using System.Globalization;
using System.Threading;
using System.Threading.Tasks;
using IRaCIS.Core.Domain.BaseModel;
using IRaCIS.Core.Domain.Share;
using MassTransit;
using Microsoft.Extensions.Logging;
using NPOI.SS.Formula.Functions;

public class ConsumeExceptionFilter<T>(ILogger<ConsumeExceptionFilter<T>> _logger)  : IFilter<ConsumeContext<T>> where T : DomainEvent
{

    public async Task Send(ConsumeContext<T> context, IPipe<ConsumeContext<T>> next)
    {

        try
        {
            await next.Send(context);

        }
        catch (Exception exception)
        {
            var errorInfo = $"Exception: {exception.Message}[{exception.StackTrace}]" + (exception.InnerException != null ? $" InnerException: {exception.InnerException.Message}[{exception.InnerException.StackTrace}]" : "");
            _logger.LogError(errorInfo);
        }
    }

    public void Probe(ProbeContext context)
    {
        context.CreateFilterScope("ConsumeException");
    }
}
